/***************************************************************************
 * First Cut at Ports0 interface: 
 * Author: Neel Sundaresan, Indiana University
 ***************************************************************************/

#ifndef _INLINES_INCLUDED
#define _INLINES_INCLUDED

#include "local_inlines.h"

#ifdef _SINGLE_PC_PER_SD
#define ll_CurrentContextId() 0
#else
#ifdef _SGI
#define ll_CurrentContextId() (((int*)(PRDA)->unused)[1])
#else
CurrentContextId() not defined
#endif
#endif

#ifdef _SINGLE_PROCESSOR_PER_PC
#define ll_myBarrierPtr() 0
#else
#define ll_myBarrierPtr() myBarrierPtr()
#endif

#ifdef _SINGLE_PC_PER_SD
#define ll_CurrentContext() (&subdomainData.onlyContext)
#else
#ifdef _SGI
#define ll_CurrentContext() (&subdomainData.contextList[ll_CurrentContextId()])
#else
#define ll_CurrentContext() not defined
#endif
#endif

#ifdef _SINGLE_PC_PER_SD
#define ll_Context(i) (&subdomainData.onlyContext)
#else
#define ll_Context(i) (&subdomainData.contextList[(i)])
#endif


#ifdef _SINGLE_SD_PER_D
#define ll_MyNodeNum() 0
#else
#define ll_MyNodeNum() (domainData.myNodeNum)
#endif

#ifdef _SINGLE_SD_PER_D
#define ll_NumNodes() 1
#else
#define ll_NumNodes() (domainData.numNodes)
#endif

/* inlining provides quite a speed-up especially for these frequently 
   called functions */

/* this accounts for atmost 256 locks of any type per thread */
/* one unsigned byte is allotted per lock */

#define LOCAL_LOCK_TYPE          0x1
#define SHARED_LOCK_TYPE         0x100
#define SPINLOCK_TYPE            0x10000

#define LOCK_REGION(lock_type) (0xFF*(x))

#define doesHaveLock(t, lock_type) ((t)->locks_info & LOCK_REGION(lock_type))
#define doesThreadHaveLocks(t) ((t)->locks_info)
#define incrHasLock(t,lock_type) ((t)->locks_info += (lock_type))
#define decrHasLock(t,lock_type) ((t)->locks_info -= (lock_type))
#define clearHasLocks(t)        ((t)->locks_info = 0x0)


#ifdef _SGI
#define ll_NumContexts() (subdomainData.numContexts)
#define ll_NumProcessors() (ll_CurrentContext()->numProcessors)
#define ll_MyProcessorNum() (((int*)(PRDA)->unused)[0])
#define ll_GetMyMemPtr() (ll_CurrentContextId() == -1 ? subdomainData.mem : ll_CurrentContext()->sharedMem)
#else
#ifdef _PARAGON
#define ll_NumContexts() 1
#define ll_NumProcessors() 1
#define ll_MyProcessorNum() 0  
#define ll_GetMyMemPtr()  0 
#endif
#endif

#ifdef _SINGLE_PROCESSOR_PER_PC
#define ll_ThisThread() (ll_CurrentContext()->thisThread_ptr)
#else
#define ll_ThisThread() (ll_CurrentContext()->thisThreadList[ll_MyProcessorNum()])
#endif
#define ll_ThisThreadId() (ThreadGetRegId(ll_ThisThread()))


#endif /* _INLINES_INCLUDED */
