/***************************************************************************
 * First Cut at Ports0 interface: 
 * Author: Neel Sundaresan, Indiana University
 ***************************************************************************/

#include "ports0.h"
extern int saveArgc;
extern char** saveArgv;

int ports0_init(int argc, char**argv, char*package_id)
{
    saveArgv = argv;
    saveArgc = argc;

    InitPCPackage();
    CreateContexts(1, 1);
    FireupContexts();
    return 0;
}

int ports0_shutdown(void)
{
#ifdef PORTS0_DEBUG
   printf("ports0_shutdown!\n");
#endif
  nodeExit(0);
  return 0;
}

/*----------------------- thread attributes ----------------------------- */
int ports0_threadattr_init(ports0_threadattr_t *attr)
{
#ifdef PORTS0_DEBUG
  printf("ports0_threadattr_init called - no op!\n");
#endif
  *attr = 0;
  return 0;
}

int ports0_threadattr_destroy(ports0_threadattr_t* attr)
{
#ifdef PORTS0_DEBUG
   printf("ports0_threadattr_destroy called - no op!\n");
#endif
   return 0;
}

int ports0_threadattr_setstacksize(ports0_threadattr_t* attr,
				   size_t stacksize)
{
#ifdef PORTS0_DEBUG
   printf("ports0_threadattr_setstacksize, stack is fixed!\n");
#endif
   *attr = stacksize;  
   return 0;
}

int ports0_threadattr_getstacksize(ports0_threadattr_t* attr,
				   size_t* stacksize)
{
   *stacksize = *attr;
   return 0;
}

/* ----------------------- thread management ------------------------------ */
int ports0_thread_create(ports0_thread_t* tp,
	                 const ports0_threadattr_t* attr,
	  	         void (*start)(void*),
                         void *arg)
{
   int stack_size;
   assert(tp);

   if (!attr)
      stack_size = ThreadStackSize;
   else  
      stack_size = (int)attr;
   *tp = allocate_thread(0, stack_size);
   threadStart(*tp, (ThreadFunction)start, arg, -1, -1);
   return 0;
}


int ports0_thread_once(ports0_thread_once_t *once_control,
                               void (*start)(void))
{
#ifdef PORTS0_DEBUG
   printf("ports0_thread_once: no op!\n");
#endif
   return 0;
}

int ports0_thread_exit(void* value)
{
   threadExit(value);
/* pthread_exit is a void function.. donno why ports0_thread_exit is an int 
   function - Neel */
   return 0;
}

int ports0_thread_yield(void)
{
  threadYield();
  return 0;
}

ports0_thread_t ports0_thread_self(void)
{
  return ll_ThisThread();
}

int ports0_thread_equal(ports0_thread_t t1, ports0_thread_t t2)
{
   return t1 == t2;
}  


/*---------------------- thread-specific data -------------------------------*/
int ports0_thread_key_create(ports0_thread_key_t* key,
	                     void (*destructor)(void*))
{
#ifdef PORTS0_DEBUG
   printf("ports0_thread_key_create: ignoring destructor!\n");
#endif
   return thread_keycreate((thread_key_t*)key);
}

int ports0_thread_key_delete(ports0_thread_key_t key)
{
#ifdef PORTS0_DEBUG
   printf("ports0_thread_key_delete: no op!\n");
#endif
   return 0;
}

void* ports0_thread_getspecific(ports0_thread_key_t key)
{
   thread_data_t val;
   int stat = thread_getspecific(ll_ThisThread(), key, &val);
   return (void*)val;
}

int ports0_thread_setspecific(ports0_thread_key_t key,
	                      const void* value)
{
    return thread_setspecific(ll_ThisThread(), key, (thread_data_t)value);
}


/*-------------------------------- mutexes ---------------------------------*/
int ports0_mutexattr_init(ports0_mutexattr_t* attr)
{
#ifdef PORTS0_DEBUG
     printf("ports0_mutexattr_init: no op!\n");
#endif
    return 0;
}

int ports0_mutexattr_destroy(ports0_mutexattr_t* attr)
{
#ifdef PORTS0_DEBUG    
     printf("ports0_mutexattr_destroy: no op!\n");
#endif
    return 0;
}

int ports0_mutex_init(ports0_mutex_t *mutex_p, ports0_mutexattr_t *attr)
{
   PCMutexInit(0, mutex_p);
   return 0;
}

int ports0_mutex_destroy (ports0_mutex_t* mutex_p)
{
    assert(mutex_p);
    assert(*mutex_p);
    PCMutexDestroy(*mutex_p);
    return 0;
}

int ports0_mutex_lock(ports0_mutex_t *mutex)
{
   PCMutexAcquire(*mutex);
   return 0;
}

int ports0_mutex_trylock(ports0_mutex_t *mutex)
{
#ifdef PORTS0_DEBUG
   printf("ports0_mutex_trylock, not implemented yet!\n");
#endif
   assert(0);
   return -1;
}


int ports0_mutex_unlock (ports0_mutex_t *mutex)
{
   PCMutexRelease(*mutex);
   return 0;
}

/*--------------------- condition variable -------------------------------*/
int ports0_condattr_init(ports0_condattr_t *attr)
{
#ifdef PORTS0_DEBUG
    printf("ports0_condattr_init, no op!\n");
#endif
    return 0;
}

int ports0_condattr_destroy(ports0_condattr_t* attr)
{
#ifdef PORTS0_DEBUG
    printf("ports0_condattr_destroy, no op!\n");
#endif
    return 0;
}
   
int ports0_cond_init(ports0_cond_t* cond_p,
	             ports0_condattr_t* attr)
{
   assert(*attr == 0);
   PCConditionInit(0, cond_p);
   return 0;
}

int ports0_cond_broadcast(ports0_cond_t *cond_p)
{
   assert(cond_p);
   PCConditionBroadcast(*cond_p);
   return 0;
}

int ports0_cond_signal (ports0_cond_t *cond_p)
{
   PCConditionSignal(*cond_p);
   return 0;
} 

int ports0_cond_wait (ports0_cond_t *cond_p,
                      ports0_mutex_t *mutex_p)
{
   assert(cond_p);
   assert(mutex_p);
   PCConditionWait(*cond_p, *mutex_p);
   return 0;
}

/*----------------------- reentrant functions   ---------------------------*/
int ports0_reentrant_lock(void)
{
   assert(0);
   return -1;
}

int ports0_reentrant_unlock (void)
{
   assert(0);
   return -1;
}

void *ports0_malloc (size_t bytes)
{
   return malloc(bytes);
}
void *ports0_realloc (void *p, size_t bytes)
{
   return realloc(p, bytes);
}

void *ports0_calloc (size_t nobj, size_t bytes)
{
   return calloc(nobj, bytes);
}

int ports0_free(void* ptr)
{
   free(ptr);
   return 0;
}

int ports0_open (char *path, int flags, int mode)
{
   return open(path, flags, mode);
}

int ports0_close (int fd)
{
   return close(fd);
} 

int ports0_read (int fd, char *buf, int nbytes)
{
   return read(fd, buf, nbytes);
}

int ports0_write (int fd, char *buf, int nbytes)
{
   return write(fd, buf, nbytes);
}

int ports0_fstat (int fd, struct stat *buf)
{
   return fstat(fd, buf);
}

int ports0_lseek (int fd, off_t offset, int whence)
{
   return lseek(fd, offset, whence);
}

/*---------the following is not in the ports0 yet! -----------------------*/
int ports0_thread_join(ports0_thread_t* tp, void** ret_val_p)
{
   assert(tp);
   threadJoin(*tp, ret_val_p);
   return 0;
}
	


 
